@@ -1,10 +1,9 @@
 """Main cloudwanderer module."""
+from collections import namedtuple
 import logging
 from botocore import xform_name
-from botocore.exceptions import ClientError
-from boto3.exceptions import ResourceNotExistsError
 import boto3
-from .custom_resource_definitions import CustomResourceDefinitions
+from .boto3_interface import CloudWandererBoto3Interface
 from .aws_urn import AwsUrn
 GLOBAL_SERVICE_REGIONAL_RESOURCE = [
     {
@@ -13,111 +12,19 @@ GLOBAL_SERVICE_REGIONAL_RESOURCE = [
 ]
 
 
-class CloudWandererBoto3Interface():
-    """Class of methods which expect boto3 resources and services rather than resource names and service names."""
-
-    def __init__(self):
-        """Class of methods which expect boto3 resources and services rather than resource names and service names."""
-        self.custom_resource_definitions = CustomResourceDefinitions().load_custom_resource_definitions()
-        self.custom_resource_attribute_definitions = CustomResourceDefinitions(
-            definition_path='attribute_definitions'
-        ).load_custom_resource_definitions()
-
-    def _get_available_services(self):
-        return boto3.Session().get_available_resources()
-
-    def get_all_resource_services(self):
-        """Return all the boto3 service Resource objects that are available, both built-in and custom."""
-        for service_name in self._get_available_services():
-            yield self.get_boto3_resource_service(service_name)
-        for service_name in self.custom_resource_definitions:
-            yield self.get_custom_resource_service(service_name)
-
-    def get_boto3_resource_service(self, service_name):
-        """Return the boto3 service Resource object matching this service_name."""
-        try:
-            return boto3.resource(service_name)
-        except ResourceNotExistsError:
-            return None
-
-    def get_custom_resource_service(self, service_name):
-        """Get the custom resource definition matching this service name."""
-        return self.custom_resource_definitions.get(service_name)
-
-    def get_resource_service_by_name(self, service_name):
-        """Return all services matching name, boto3 or custom."""
-        boto3_resource_service = self.get_boto3_resource_service(service_name)
-        if boto3_resource_service:
-            yield boto3_resource_service
-        custom_resource_service = self.get_custom_resource_service(service_name)
-        if custom_resource_service:
-            yield custom_resource_service
-
-    def get_resource_collections(self, boto3_service):
-        """Return all resource types in this service."""
-        return boto3_service.meta.resource_model.collections
-
-    def get_resource_from_collection(self, boto3_service, boto3_resource_collection):
-        """Return all resources of this resource type (collection) from this service."""
-        logging.info(f'--> Fetching {boto3_service.meta.service_name} {boto3_resource_collection.name}')
-        try:
-            for resource in getattr(boto3_service, boto3_resource_collection.name).all():
-                yield resource
-        except ClientError as ex:
-            if ex.response['Error']['Code'] == 'InvalidAction':
-                logging.warning(ex.response['Error']['Message'])
-                return
-            raise ex
-
-    def get_resource_attribute_from_collection(
-            self, boto3_resource_attribute_service, boto3_resource_attribute_collection):
-        """Return a boto3.resource pertaining to a resource attribute defined by CloudWanderer.
-
-        These custom resource attribute definitions allow us to fetch resource attributes that are not returned by the
-        resource's default describe calls.
-
-        Arguments:
-            boto3_resource_attribute_service: The boto3.resource service from
-                self.get_resource_attributes_service_by_name
-            boto3_resource_attribute_collection: The boto3 collection of attributes we want to retrieve.
-        """
-        resource_attributes = self.get_resource_from_collection(
-            boto3_service=boto3_resource_attribute_service,
-            boto3_resource_collection=boto3_resource_attribute_collection
-        )
-        for resource_attribute in resource_attributes:
-            # A resource_attribute will initially be populated with its parent resource's data,
-            # the attribute data is loaded on .load()
-            resource_attribute.load()
-            # I'm fairly sure there must be a way to clean out the response metadata with botocore shapes but
-            # I haven't figured out how yet.
-            if 'ResponseMetadata' in resource_attribute.meta.data:
-                del resource_attribute.meta.data['ResponseMetadata']
-            yield resource_attribute
-
-    def get_resource_attributes_service_by_name(self, service_name):
-        """Return the boto3.resource service containing a collection of resource attributes provided by CloudWanderer.
-
-        These custom resource attribute definitions allow us to fetch resource attributes that are not returned by the
-        resource's default describe calls.
-
-        Arguments:
-            service_name (str): The name of the service (e.g.``ec2``) to get.
-        """
-        yield self.custom_resource_attribute_definitions.get(service_name)
-
-
 class CloudWanderer():
     """CloudWanderer.
 
     Args:
         storage_connector: A CloudWanderer storage connector object.
+        boto3_session (boto3.session.Session): A boto3 Session object.
     """
 
-    def __init__(self, storage_connector):
+    def __init__(self, storage_connector, boto3_session=None):
         """Initialise CloudWanderer."""
         self.storage_connector = storage_connector
-        self.boto3_interface = CloudWandererBoto3Interface()
+        self.boto3_session = boto3_session or boto3.session.Session()
+        self.boto3_interface = CloudWandererBoto3Interface(boto3_session=self.boto3_session)
         self._account_id = None
         self._client_region = None
 
@@ -125,16 +32,10 @@ class CloudWanderer():
         """Write all AWS resources in this account from all services to storage."""
         exclude_resources = exclude_resources or []
         for boto3_service in self.boto3_interface.get_all_resource_services():
-            for boto3_resource_collection in self.boto3_interface.get_resource_collections(boto3_service):
-                if boto3_resource_collection.name in exclude_resources:
-                    logging.info('Skipping %s as per exclude_resources', boto3_resource_collection.name)
-                    continue
-                resources = self.boto3_interface.get_resource_from_collection(
-                    boto3_service,
-                    boto3_resource_collection
-                )
-                for boto3_resource in resources:
-                    self.storage_connector.write_resource(self._get_resource_urn(boto3_resource), boto3_resource)
+            self.write_resources(
+                service_name=boto3_service.meta.service_name,
+                exclude_resources=exclude_resources
+            )
 
     def write_resources(self, service_name, exclude_resources=None):
         """Write all AWS resources in this account in this service to storage.
@@ -150,6 +51,7 @@ class CloudWanderer():
                 if boto3_resource_collection.name in exclude_resources:
                     logging.info('Skipping %s as per exclude_resources', boto3_resource_collection.name)
                     continue
+                logging.info(f'--> Fetching {boto3_service.meta.service_name} {boto3_resource_collection.name}')
                 resources = self.boto3_interface.get_resource_from_collection(boto3_service, boto3_resource_collection)
                 for boto3_resource in resources:
                     self.storage_connector.write_resource(self._get_resource_urn(boto3_resource), boto3_resource)
@@ -167,6 +69,11 @@ class CloudWanderer():
         for boto3_resource_attribute_service in services:
             collections = self.boto3_interface.get_resource_collections(boto3_resource_attribute_service)
             for boto3_resource_attribute_collection in collections:
+                logging.info(
+                    '--> Fetching %s %s',
+                    boto3_resource_attribute_service.meta.service_name,
+                    boto3_resource_attribute_collection.name
+                )
                 resource_attributes = self.boto3_interface.get_resource_attribute_from_collection(
                     boto3_resource_attribute_service,
                     boto3_resource_attribute_collection
@@ -204,17 +111,17 @@ class CloudWanderer():
 
     @property
     def account_id(self):
-        """Return the AWS Account ID our boto3 client is authenticated against."""
+        """Return the AWS Account ID our boto3 session is authenticated against."""
         if self._account_id is None:
-            sts = boto3.client('sts')
+            sts = self.boto3_session.client('sts')
             self._account_id = sts.get_caller_identity()['Account']
         return self._account_id
 
     @property
     def client_region(self):
-        """Return the region our boto3 client is authenticated against."""
+        """Return the region our boto3 session is configured to use."""
         if self._client_region is None:
-            self._client_region = boto3.session.Session().region_name
+            self._client_region = self.boto3_session.region_name
         return self._client_region
 
     def _get_resource_urn(self, resource):
@@ -231,8 +138,19 @@ class CloudWanderer():
         )
 
 
-class ResourceDict(dict):
-    """A dictionary representation of a resource that prevents any storage metadata polluting the resource dictionary.
+class ResourceMetadata(namedtuple('ResourceMetadata', ['resource_data', 'resource_attributes'])):
+    """Metadata for a :class:`CloudWandererResource`.
+
+    Contains the original dictionaries of the resource and its attributes.
+
+    Attributes:
+        resource_data (dict): The raw dictionary representation of the Resource.
+        resource_attributes (list): the list of raw dictionary representation of the Resource's attributes.
+    """
+
+
+class CloudWandererResource():
+    """A simple representation of a resource that prevents any storage metadata polluting the resource dictionary.
 
     Use ``dict(my_resource_dict)`` to convert this object into a dictionary that
     contains *only* the resource's metadata.
@@ -242,11 +160,14 @@ class ResourceDict(dict):
         metadata (dict): The original storage representation of the resource as it was passed in.
     """
 
-    def __init__(self, urn, resource):
-        """Initialise the resource dict."""
+    def __init__(self, urn, resource_data, resource_attributes=None):
+        """Initialise the resource."""
         self.urn = urn
-        self.metadata = resource
-        super().__init__(**self._clean_resource)
+        self.cloudwanderer_metadata = ResourceMetadata(
+            resource_data=resource_data,
+            resource_attributes=resource_attributes or []
+        )
+        self._set_attrs()
 
     @property
     def _clean_resource(self):
@@ -256,9 +177,22 @@ class ResourceDict(dict):
             if not key.startswith('_')
         }
 
+    def _set_attrs(self):
+        for key, value in self.cloudwanderer_metadata.resource_data.items():
+            setattr(
+                self,
+                xform_name(key),
+                value
+            )
+
     def __repr__(self):
         """Return a code representation of this resource."""
-        return f"{self.__class__.__name__}(urn={repr(self.urn)}, resource={self.metadata})"
+        return str(
+            f"{self.__class__.__name__}("
+            f"urn={repr(self.urn)}, "
+            f"resource_data={self.cloudwanderer_metadata.resource_data}, "
+            f"resource_attributes={self.cloudwanderer_metadata.resource_attributes})"
+        )
 
     def __str__(self):
         """Return the string representation of this Resource."""
